@@ -21,7 +21,6 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <string.h>
 #include <stdlib.h>
 #include "scrollarea.h"
@@ -59,6 +58,10 @@ struct App
     GtkWidget	   *clone_checkbox;
     GtkWidget	   *show_icon_checkbox;
 
+    /* We store the event timestamp when the Apply button is clicked */
+    GtkWidget      *apply_button;
+    guint32         apply_button_clicked_timestamp;
+
     GtkWidget      *area;
     gboolean	    ignore_gui_changes;
     GConfClient	   *client;
@@ -67,6 +70,11 @@ struct App
     DBusGConnection *connection;
     DBusGProxy *proxy;
     DBusGProxyCall *proxy_call;
+
+    enum {
+	APPLYING_VERSION_1,
+	APPLYING_VERSION_2
+    } apply_configuration_state;
 };
 
 static void rebuild_gui (App *app);
@@ -75,6 +83,7 @@ static gboolean output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config)
 static void select_current_output_from_dialog_position (App *app);
 static void monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data);
 static void get_geometry (GnomeOutputInfo *output, int *w, int *h);
+static void apply_configuration_returned_cb (DBusGProxy *proxy, DBusGProxyCall *call_id, void *data);
 
 static void
 error_message (App *app, const char *primary_text, const char *secondary_text)
@@ -86,7 +95,9 @@ error_message (App *app, const char *primary_text, const char *secondary_text)
 				     GTK_MESSAGE_ERROR,
 				     GTK_BUTTONS_CLOSE,
 				     "%s", primary_text);
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary_text);
+
+    if (secondary_text)
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary_text);
 
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -469,8 +480,26 @@ rebuild_current_monitor_label (App *app)
 	    g_free (str);
 
 	if (use_color)
+	{
+	    GdkColor black = { 0, 0, 0, 0 };
+
 	    gtk_widget_modify_bg (app->current_monitor_event_box, app->current_monitor_event_box->state, &color);
 
+	    /* Make the label explicitly black.  We don't want it to follow the
+	     * theme's colors, since the label is always shown against a light
+	     * pastel background.  See bgo#556050
+	     */
+	    gtk_widget_modify_fg (app->current_monitor_label, GTK_WIDGET_STATE (app->current_monitor_label), &black);
+	}
+	else
+	{
+	    /* Remove any modifications we did on the label's color */
+	    GtkRcStyle *reset_rc_style;
+
+	    reset_rc_style = gtk_rc_style_new ();
+	    gtk_widget_modify_style (app->current_monitor_label, reset_rc_style); /* takes ownership of, and destroys, the rc style */
+	}
+
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (app->current_monitor_event_box), use_color);
 }
 
@@ -792,10 +821,11 @@ lay_out_outputs_horizontally (App *app)
 	GnomeOutputInfo *output;
 
 	output = app->current_configuration->outputs[i];
-	if (output->connected && output->on)
+	if (output->connected && output->on) {
 	    output->x = x;
-
-	x += output->width;
+	    output->y = 0;
+	    x += output->width;
+	}
     }
 
     /* Second pass, all the black screens */
@@ -805,10 +835,11 @@ lay_out_outputs_horizontally (App *app)
 	GnomeOutputInfo *output;
 
 	output = app->current_configuration->outputs[i];
-	if (!(output->connected && output->on))
+	if (!(output->connected && output->on)) {
 	    output->x = x;
-
-	x += output->width;
+	    output->y = 0;
+	    x += output->width;
+	}
     }
 
 }
@@ -1711,6 +1742,79 @@ check_required_virtual_size (App *app)
     }
 }
 
+static void
+begin_version2_apply_configuration (App *app, GdkWindow *parent_window, guint32 timestamp)
+{
+    XID parent_window_xid;
+
+    parent_window_xid = GDK_WINDOW_XID (parent_window);
+
+    app->proxy = dbus_g_proxy_new_for_name (app->connection,
+					    "org.gnome.SettingsDaemon",
+					    "/org/gnome/SettingsDaemon/XRANDR",
+					    "org.gnome.SettingsDaemon.XRANDR_2");
+    g_assert (app->proxy != NULL); /* that call does not fail unless we pass bogus names */
+
+    app->apply_configuration_state = APPLYING_VERSION_2;
+    app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
+					       apply_configuration_returned_cb, app,
+					       NULL,
+					       G_TYPE_INT64, (gint64) parent_window_xid,
+					       G_TYPE_INT64, (gint64) timestamp,
+					       G_TYPE_INVALID,
+					       G_TYPE_INVALID);
+    /* FIXME: we don't check for app->proxy_call == NULL, which could happen if
+     * the connection was disconnected.  This is left as an exercise for the
+     * reader.
+     */
+}
+
+static void
+begin_version1_apply_configuration (App *app)
+{
+    app->proxy = dbus_g_proxy_new_for_name (app->connection,
+					    "org.gnome.SettingsDaemon",
+					    "/org/gnome/SettingsDaemon/XRANDR",
+					    "org.gnome.SettingsDaemon.XRANDR");
+    g_assert (app->proxy != NULL); /* that call does not fail unless we pass bogus names */
+
+    app->apply_configuration_state = APPLYING_VERSION_1;
+    app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
+					       apply_configuration_returned_cb, app,
+					       NULL,
+					       G_TYPE_INVALID,
+					       G_TYPE_INVALID);
+    /* FIXME: we don't check for app->proxy_call == NULL, which could happen if
+     * the connection was disconnected.  This is left as an exercise for the
+     * reader.
+     */
+}
+
+static void
+ensure_current_configuration_is_saved (void)
+{
+        GnomeRRScreen *rr_screen;
+        GnomeRRConfig *rr_config;
+
+        /* Normally, gnome_rr_config_save() creates a backup file based on the
+         * old monitors.xml.  However, if *that* file didn't exist, there is
+         * nothing from which to create a backup.  So, here we'll save the
+         * current/unchanged configuration and then let our caller call
+         * gnome_rr_config_save() again with the new/changed configuration, so
+         * that there *will* be a backup file in the end.
+         */
+
+        rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), NULL, NULL, NULL); /* NULL-GError */
+        if (!rr_screen)
+                return;
+
+        rr_config = gnome_rr_config_new_current (rr_screen);
+        gnome_rr_config_save (rr_config, NULL); /* NULL-GError */
+
+        gnome_rr_config_free (rr_config);
+        gnome_rr_screen_destroy (rr_screen);
+}
+
 /* Callback for dbus_g_proxy_begin_call() */
 static void
 apply_configuration_returned_cb (DBusGProxy       *proxy,
@@ -1727,8 +1831,21 @@ apply_configuration_returned_cb (DBusGProxy       *proxy,
     success = dbus_g_proxy_end_call (proxy, call_id, &error, G_TYPE_INVALID);
 
     if (!success) {
-	error_message (app, _("Could not apply the selected configuration"), error->message);
-	g_error_free (error);
+	if (app->apply_configuration_state == APPLYING_VERSION_2
+	    && g_error_matches (error, DBUS_GERROR, DBUS_GERROR_UNKNOWN_METHOD)) {
+	    g_error_free (error);
+
+	    g_object_unref (app->proxy);
+	    app->proxy = NULL;
+
+	    begin_version1_apply_configuration (app);
+	    return;
+	} else {
+	    /* We don't pop up an error message; gnome-settings-daemon already does that
+	     * in case the selected RANDR configuration could not be applied.
+	     */
+	    g_error_free (error);
+	}
     }
 
     g_object_unref (app->proxy);
@@ -1752,6 +1869,8 @@ apply (App *app)
 
     foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
 
+    ensure_current_configuration_is_saved ();
+
     if (!gnome_rr_config_save (app->current_configuration, &error))
     {
 	error_message (app, _("Could not save the monitor configuration"), error->message);
@@ -1770,23 +1889,9 @@ apply (App *app)
 	return;
     }
 
-    app->proxy = dbus_g_proxy_new_for_name (app->connection,
-					    "org.gnome.SettingsDaemon",
-					    "/org/gnome/SettingsDaemon/XRANDR",
-					    "org.gnome.SettingsDaemon.XRANDR");
-    if (!app->proxy) {
-	error_message (app, _("Could not get org.gnome.SettingsDaemon.XRANDR"), NULL);
-	dbus_g_connection_unref (app->connection);
-	app->connection = NULL;
-	return;
-    }
-
     gtk_widget_set_sensitive (app->dialog, FALSE);
-    app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
-					       apply_configuration_returned_cb, app,
-					       NULL,
-					       G_TYPE_INVALID,
-					       G_TYPE_INVALID);
+
+    begin_version2_apply_configuration (app, gtk_widget_get_window (app->dialog), app->apply_button_clicked_timestamp);
 }
 
 #if 0
@@ -1991,38 +2096,60 @@ hide_help_button (App *app)
     }
 }
 
+static void
+apply_button_clicked_cb (GtkButton *button, gpointer data)
+{
+    App *app = data;
+
+    /* We simply store the timestamp at which the Apply button was clicked.
+     * We'll just wait for the dialog to return from gtk_dialog_run(), and
+     * *then* use the timestamp when applying the RANDR configuration.
+     */
+
+    app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
+}
+
+static GtkWidget*
+_gtk_builder_get_widget (GtkBuilder *builder, const gchar *name)
+{
+    return GTK_WIDGET (gtk_builder_get_object (builder, name));
+}
+
 static void
 run_application (App *app)
 {
-#ifndef GLADEDIR
-#define GLADEDIR "."
+#ifndef UIDIR
+#define UIDIR "."
 #endif
-#define GLADE_FILE GLADEDIR "/display-capplet.glade"
-    GladeXML *xml;
+#define UI_FILE UIDIR "/display-capplet.ui"
+    GtkBuilder *builder;
     GtkWidget *align;
     GError *error;
 
-    xml = glade_xml_new (GLADE_FILE, NULL, NULL);
-    if (!xml)
+    error = NULL;
+    builder = gtk_builder_new ();
+
+    if (gtk_builder_add_from_file (builder, UI_FILE, &error) == 0)
     {
-	g_warning ("Could not open " GLADE_FILE);
+	g_warning ("Could not parse UI definition: %s", error->message);
+	g_error_free (error);
+	g_object_unref (builder);
 	return;
     }
 
-    error = NULL;
     app->screen = gnome_rr_screen_new (gdk_screen_get_default (),
 				       on_screen_changed, app, &error);
     if (!app->screen)
     {
 	error_message (NULL, _("Could not get screen information"), error->message);
 	g_error_free (error);
-	g_object_unref (xml);
+	g_object_unref (builder);
 	return;
     }
 
     app->client = gconf_client_get_default ();
 
-    app->dialog = glade_xml_get_widget (xml, "dialog");
+    app->dialog = _gtk_builder_get_widget (builder, "dialog");
     g_signal_connect_after (app->dialog, "map-event",
 			    G_CALLBACK (dialog_map_event_cb), app);
 
@@ -2030,43 +2157,49 @@ run_application (App *app)
     gtk_window_set_icon_name (GTK_WINDOW (app->dialog),
 			      "gnome-display-properties");
 
-    app->current_monitor_event_box = glade_xml_get_widget (xml, "current_monitor_event_box");
-    app->current_monitor_label = glade_xml_get_widget (xml, "current_monitor_label");
+    app->current_monitor_event_box = _gtk_builder_get_widget (builder,
+    						   "current_monitor_event_box");
+    app->current_monitor_label = _gtk_builder_get_widget (builder,
+    						       "current_monitor_label");
 
-    app->monitor_on_radio = glade_xml_get_widget (xml, "monitor_on_radio");
-    app->monitor_off_radio = glade_xml_get_widget (xml, "monitor_off_radio");
+    app->monitor_on_radio = _gtk_builder_get_widget (builder,
+    						     "monitor_on_radio");
+    app->monitor_off_radio = _gtk_builder_get_widget (builder,
+    						      "monitor_off_radio");
     g_signal_connect (app->monitor_on_radio, "toggled",
 		      G_CALLBACK (monitor_on_off_toggled_cb), app);
     g_signal_connect (app->monitor_off_radio, "toggled",
 		      G_CALLBACK (monitor_on_off_toggled_cb), app);
 
-    app->resolution_combo = glade_xml_get_widget (xml, "resolution_combo");
+    app->resolution_combo = _gtk_builder_get_widget (builder,
+    						     "resolution_combo");
     g_signal_connect (app->resolution_combo, "changed",
 		      G_CALLBACK (on_resolution_changed), app);
 
-    app->refresh_combo = glade_xml_get_widget (xml, "refresh_combo");
+    app->refresh_combo = _gtk_builder_get_widget (builder, "refresh_combo");
     g_signal_connect (app->refresh_combo, "changed",
 		      G_CALLBACK (on_rate_changed), app);
 
-    app->rotation_combo = glade_xml_get_widget (xml, "rotation_combo");
+    app->rotation_combo = _gtk_builder_get_widget (builder, "rotation_combo");
     g_signal_connect (app->rotation_combo, "changed",
 		      G_CALLBACK (on_rotation_changed), app);
 
-    app->clone_checkbox = glade_xml_get_widget (xml, "clone_checkbox");
+    app->clone_checkbox = _gtk_builder_get_widget (builder, "clone_checkbox");
     g_signal_connect (app->clone_checkbox, "toggled",
 		      G_CALLBACK (on_clone_changed), app);
 
-    g_signal_connect (glade_xml_get_widget (xml, "detect_displays_button"),
+    g_signal_connect (_gtk_builder_get_widget (builder, "detect_displays_button"),
 		      "clicked", G_CALLBACK (on_detect_displays), app);
 
-    app->show_icon_checkbox = glade_xml_get_widget (xml, "show_notification_icon");
+    app->show_icon_checkbox = _gtk_builder_get_widget (builder,
+						      "show_notification_icon");
 
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->show_icon_checkbox),
 				  gconf_client_get_bool (app->client, SHOW_ICON_KEY, NULL));
 
     g_signal_connect (app->show_icon_checkbox, "toggled", G_CALLBACK (on_show_icon_toggled), app);
 
-    app->panel_checkbox = glade_xml_get_widget (xml, "panel_checkbox");
+    app->panel_checkbox = _gtk_builder_get_widget (builder, "panel_checkbox");
 
     make_text_combo (app->resolution_combo, 4);
     make_text_combo (app->refresh_combo, 3);
@@ -2087,16 +2220,20 @@ run_application (App *app)
     g_signal_connect (app->area, "viewport_changed",
 		      G_CALLBACK (on_viewport_changed), app);
 
-    align = glade_xml_get_widget (xml, "align");
+    align = _gtk_builder_get_widget (builder, "align");
 
     gtk_container_add (GTK_CONTAINER (align), app->area);
 
     /* Until we have help to show, we'll just hide the Help button */
     hide_help_button (app);
 
+    app->apply_button = _gtk_builder_get_widget (builder, "apply_button");
+    g_signal_connect (app->apply_button, "clicked",
+		      G_CALLBACK (apply_button_clicked_cb), app);
+
     on_screen_changed (app->screen, app);
 
-    g_object_unref (xml);
+    g_object_unref (builder);
 
 restart:
     switch (gtk_dialog_run (GTK_DIALOG (app->dialog)))
