@@ -18,15 +18,20 @@
  * Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#undef GTK_DISABLE_DEPRECATED
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnomeui/libgnomeui.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
+#include <gio/gio.h>
+#include <gdk/gdkkeysyms.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <string.h>
+#include <stdlib.h>
 
 #include <glib/gi18n-lib.h>
 
@@ -649,14 +654,14 @@ delete_old_data (AppShellData * app_data)
 		CategoryData *data = (CategoryData *) cat_list->data;
 		gtk_widget_destroy (GTK_WIDGET (data->section));
 		gtk_widget_destroy (GTK_WIDGET (data->group_launcher));
-		gtk_object_unref (GTK_OBJECT (data->section));
-		gtk_object_unref (GTK_OBJECT (data->group_launcher));
+		g_object_unref (data->section);
+		g_object_unref (data->group_launcher);
 		g_free (data->category);
 
 		for (temp = data->launcher_list; temp; temp = g_list_next (temp))
 		{
 			g_free (g_object_get_data (G_OBJECT (temp->data), TILE_EXEC_NAME));
-			gtk_object_unref (temp->data);
+			g_object_unref (temp->data);
 		}
 
 		g_list_free (data->launcher_list);
@@ -746,7 +751,7 @@ show_no_results_message (AppShellData * app_data, GtkWidget * containing_vbox)
 		gtk_container_add (GTK_CONTAINER (app_data->filtered_out_everything_widget), hbox);
 	}
 
-	markup = g_strdup_printf (
+	markup = g_markup_printf_escaped (
 		_("<span size=\"large\"><b>No matches found.</b> </span><span>\n\n Your filter \"<b>%s</b>\" does not match any items.</span>"),
 		app_data->filter_string);
 	gtk_label_set_text (app_data->filtered_out_everything_widget_label, markup);
@@ -1031,10 +1036,8 @@ generate_launchers (GMenuTreeDirectory * root_dir, AppShellData * app_data, Cate
 			desktop_item = gnome_desktop_item_new_from_file (desktop_file, 0, NULL);
 			if (!desktop_item)
 			{
-				gchar *error_msg = g_strdup_printf("Failure - gnome_desktop_item_new_from_file(%s)",
-					desktop_file);
-				g_critical (error_msg);
-				g_free (error_msg);
+				g_critical ("Failure - gnome_desktop_item_new_from_file(%s)",
+					    desktop_file);
 				break;
 			}
 			if (!check_specific_apps_hack (desktop_item))
@@ -1139,7 +1142,8 @@ generate_new_apps (AppShellData * app_data)
 			const gchar *uri = gnome_desktop_item_get_location (item);
 			if (!g_hash_table_lookup (all_apps_cache, uri))
 			{
-				GnomeVFSFileInfo *info;
+				GFile *file;
+				GFileInfo *info;
 				long filetime;
 
 				if (g_hash_table_lookup (new_apps_dups, uri))
@@ -1165,17 +1169,21 @@ generate_new_apps (AppShellData * app_data)
 					got_new_apps = TRUE;
 				}
 
-				info = gnome_vfs_file_info_new ();
-				if (gnome_vfs_get_file_info (uri, info,
-						GNOME_VFS_FILE_INFO_DEFAULT) != GNOME_VFS_OK
-					|| !(info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME))
+				file = g_file_new_for_uri (uri);
+				info = g_file_query_info (file,
+							  G_FILE_ATTRIBUTE_TIME_MODIFIED,
+							  0, NULL, NULL);
+
+				if (!info)
 				{
-					gnome_vfs_file_info_unref (info);
+					g_object_unref (file);
 					g_warning ("Cant get vfs info for %s\n", uri);
 					return;
 				}
-				filetime = info->mtime;
-				gnome_vfs_file_info_unref (info);
+				filetime = (long) g_file_info_get_attribute_uint64 (info,
+										    G_FILE_ATTRIBUTE_TIME_MODIFIED);
+				g_object_unref (info);
+				g_object_unref (file);
 
 				for (x = 0; x < app_data->new_apps->max_items; x++)
 				{
